//       1         2         3         4         5         6         7
// 4567890123456789012345678901234567890123456789012345678901234567890123456789

package la.util;

import static la.util.Util.*;  // all those methods/functions
import la.la.Value;
// ----------------------------------------------------------------------------

/* Copyright (c) 2018 Lloyd Allison.

   Reference: 
     Lloyd Allison, "Coding Ockham's Razor", Springer, 2018.

   This file is free software: you can redistribute it and/or modify it
   under the terms of the GNU General Public License (GPL) as published
   by the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This file is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this file. If not, see http://www.gnu.org/licenses/
 */
// ----------------------------------------------------------------------------

/** {@link Test#main} runs a few <I>simple</I> tests of
    {@link Util} and {@link Series}
    in package {@link README la.util}. */
public class Test
 {
   /** Return 'n' in hexadecimal as a String. */
   public static String hex(int n)
    { String ans = "";
      int stopper = -1;
      for( int i = 0; stopper != 0; i += 4 )
        { ans = java.lang.Integer.toString((n >>> i) & 15, 16) + ans;
          stopper >>>= 4;
        }
      return ans;
 // return java.lang.Integer.toString(n,16); }
    }

   /** Run a few, very simple tests. */
   public static void main(String[] args)
    { System.out.println("-- test Util --");

      double[] ds = new double[]{ 0.0,
        0.1, 0.01, 0.001, 0.0001, 0.00001, 0.000001, 0.0000001,
        1.0, 10.0, 100.0, 1000.0, 10000.0, 100000.0, 1000000.0,
        0.123456789, 0.0123456789, 0.00123456789, 0.000123456789,
        0.0000123456789,
        1.23456789, 12.3456789, 123.456789, 1234.56789, 123456.789, 1234567.89
        };

      System.out.println( "Printing doubles to various sig. figs:" );
      for( int i = 0; i < ds.length; i ++ )
       { double x = ds[i], xm = - x;
         System.out.println( "  " + x2String(x) + ", " + x2String(xm)
                           + "   (raw x=" + x + ")" );
       }//for i

      System.out.println( "Tests of interleave16(m,n) on random m and n:" );
      java.util.Random RNG = new java.util.Random();
      for( int i = 0; i < 8; i ++ )
       { final int m = RNG.nextInt(), n = RNG.nextInt();     // interleave bits
         int mn = 0,  mni = interleave16(m, n);
         for( int b = 0; b < 16; b ++ )
          { int bit = 1 << b; 
            mn |= ( ((m & bit) << 1) | (n & bit) ) << b; // "obvious" way
          }
         System.out.println( "  interleave16("
                           + hex(m) + ", " + hex(n) + ") = " + hex(mni)
                           + " ("+(mn==mni ? "ok =" : "!=") + hex(mn)+")" );
       }//for i

      System.out.println( "A test of class Series:" );
      Series s1 = new Series.Range(0, 10, 2),
             s2 = new Series.Range(0, 10, 3);
      Series s  = s1.merge(s2).append(new Series.Range(10,15));
      System.out.print("  Series s, (posn:elt())* :");
      while( s.hasSome() )
       { System.out.print(" " + s.position() + ":" + s.elt()); s.advance(); }
      System.out.println();

      System.out.println( "sl = (a Series).asList() :" );
      Value.List sl = new Series.Range(0, 10, 2).asList();
      System.out.println("   sl=" + sl + ", again sl=" + sl);

      System.out.println("-- done Util --");
    }//main

 }//Util class   http://www.allisons.org/ll/

// (c) Lloyd Allison, Oct 2011, Wed 16 Aug 2017. Licence to 'Test' is Gnu GPL.
