//       1         2         3         4         5         6         7
// 4567890123456789012345678901234567890123456789012345678901234567890123456789

package mml;

import la.util.Util;   // standard functions
import la.la.Value;
import static la.maths.Maths.*;
import la.maths.Vector;
import la.maths.Vector.Slice;

import static mml.MML.*;
// ----------------------------------------------------------------------------

/* Copyright (c) 2018 Lloyd Allison.

   Reference: 
     Lloyd Allison, "Coding Ockham's Razor", Springer, 2018.

   This file is free software: you can redistribute it and/or modify it
   under the terms of the GNU General Public License (GPL) as published
   by the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This file is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this file. If not, see http://www.gnu.org/licenses/
 */
// ----------------------------------------------------------------------------

/** (INCOMPLETE, no Estimator yet)
    The <B>&Beta;</B> (Beta) Model (probability distribution). */

public class BetaUPM extends Continuous                                 // Beta
 {
   public BetaUPM(Value t)
    { super(t); if( t != triv ) error( ", t=" + t ); }//constructor

   /** Given two-part message lengths,  m1 &amp; m2, and
  <NOBR>sp = &lang;{@link M#alpha &alpha;}, {@link M#beta &beta;}&rang;</NOBR>,
       return a fully parameterised {@link M Beta} Model. */
   public M sp2Model(double m1, double m2, Value sp)
    { return new M(m1, m2, sp); }

   /** Return elements [lo, hi) of data-set ds, itself; TODO note that
       <NOBR>&lang;&Sum; log X<SUB>i</SUB>,
                   &Sum; log(1-X<SUB>i</SUB>)&rang;</NOBR>
       would be better.  */
   public Vector stats(Vector ds, int lo, int hi)
    { return ds.slice(lo,hi); } // TODO: Is there a better one?

   public Vector stats(boolean add, Value ss0, Value ss1)
    { return ((Vector) ss0).combine(add, (Slice) ss1); }

   /** The Estimator is not yet implemented. */
   public Est estimator(Value ps)
    { throw UOE( " not yet implemented" ); }

   public String toString()
    { return this == Beta ? "Beta" : super.toString(); }
   // .......................................

   /** The fully parameterised Beta Model (probability distribution). */
   public class M extends Continuous.M                         // BetaUPM.M
    {
      /** Shape parameters 'alpha' and 'beta'. */
      public final double alpha, beta;

      /** Negative log normalising constant for {@link #nlPdf nlPdf}. */
      private final double alpha_1, beta_1, nlNormaliser;

      /** Statistical parameters,
          sp = &lang;{@link #alpha alpha}, {@link #beta beta}&rang;. */
      public M(double msg1, double msg2, Value sp)
       { super(msg1, msg2, sp);
         Value.Tuple sp2 = sp.isTuple(2);
         alpha = sp2.elt(0).x(); beta = sp2.elt(1).x();
         assert alpha > 0 && beta > 0 : "alpha=" + alpha + ", beta=" + beta;
         alpha_1 = alpha-1; beta_1 = beta-1;
         nlNormaliser = logBeta(alpha, beta);
       }//constructor

      /** pdf(x) = {1 / <B>&Beta;</B>(alpha, beta)}
                   x<SUP>alpha-1</SUP> (1-x)<SUP>beta-1</SUP>. */
      public double nlPdf_x(double x)
       { assert x >= 0 && x <= 1 : this + ".nlPdf(" + x + ")";
         return nlNormaliser - alpha_1*x - beta_1*(1-x);
       }//nlPdf(x)

      /** Assumes that {@link BetaUPM#stats(Vector,int,int) statistics}
          are the data-set itself. */
      public double nlLH(Value ss)
       { return sumNlPr((Vector) ss); }

      private GammaUPM.M G_A_1 = null, G_B_1 = null;  // see random_x() below

      /** If X ~ Gamma(alpha, theta) and <NOBR>Y ~ Gamma(beta, theta)</NOBR>
          independently then <NOBR>(X/(X+Y)) ~ <B>B</B>(alpha, beta)</NOBR>
          -- van der Waerden, <I>Mathematical Statistics</I>, Springer, 1969,
          cited in wikip.  */
      public double random_x()
       { if( G_A_1 == null )
          { G_A_1 = Gamma.sp2Model(0, 0, pair(real(alpha), one));
            G_B_1 = Gamma.sp2Model(0, 0, pair(real(beta),  one));
          }
         double X = G_A_1.random_x(), Y = G_B_1.random_x();
         return X / (X + Y );
       }//random_x()

    }//class BetaUPM.M, the fully paramaterised Model
   // -------------------------------------------------------------------------

   /** Trival tests of the <B>&Beta;</B> Model (probability distribution). */
   public static void main(String[] argv)                               // main
    { System.out.println("-- run mml.Beta --");

      Real alpha = real(2), beta = real(5);
      BetaUPM.M b = Beta.sp2Model(0, 0, pair(alpha, beta));
      System.out.println( b + ":" );
      for( int i = 0; i < 10; i ++ )
         System.out.print( b.random() + ", " );
      System.out.println();

      System.out.println("-- done mml.Beta --");
    }//main

 }//class BetaUPM, the UnParametrised Model

// (c) Lloyd Allison, 27 February 2014,  licence is Gnu GPL.
